@@ -203,9 +203,9 @@ static const struct net_device_ops sierra_net_device_ops = {
 
 	.ndo_change_mtu         = usbnet_change_mtu,
 #if LINUX_VERSION_IS_GEQ(4,11,0)
-	.ndo_get_stats64        = usbnet_get_stats64,
+	.ndo_get_stats64        = dev_get_tstats64,
 #else
-	.ndo_get_stats64 = bp_usbnet_get_stats64,
+	.ndo_get_stats64 = bp_dev_get_tstats64,
 #endif
 
 	.ndo_set_mac_address    = eth_mac_addr,
@@ -988,9 +988,7 @@ static struct usb_driver sierra_net_driver = {
 	.suspend = usbnet_suspend,
 	.resume = usbnet_resume,
 	.no_dynamic_id = 1,
-#if LINUX_VERSION_IS_GEQ(3,5,0)
 	.disable_hub_initiated_lpm = 1,
-#endif
 };
 
 module_usb_driver(sierra_net_driver);
