@@ -198,7 +198,7 @@ static void ath11k_copy_regd(struct ieee80211_regdomain *regd_orig,
 		       sizeof(struct ieee80211_reg_rule));
 }
 
-int ath11k_regd_update(struct ath11k *ar)
+int ath11k_regd_update(struct ath11k *ar, bool init)
 {
 	struct ieee80211_regdomain *regd, *regd_copy = NULL;
 	int ret, regd_len, pdev_id;
@@ -209,10 +209,7 @@ int ath11k_regd_update(struct ath11k *ar)
 
 	spin_lock_bh(&ab->base_lock);
 
-	/* Prefer the latest regd update over default if it's available */
-	if (ab->new_regd[pdev_id]) {
-		regd = ab->new_regd[pdev_id];
-	} else {
+	if (init) {
 		/* Apply the regd received during init through
 		 * WMI_REG_CHAN_LIST_CC event. In case of failure to
 		 * receive the regd, initialize with a default world
@@ -225,6 +222,8 @@ int ath11k_regd_update(struct ath11k *ar)
 				    "failed to receive default regd during init\n");
 			regd = (struct ieee80211_regdomain *)&ath11k_world_regd;
 		}
+	} else {
+		regd = ab->new_regd[pdev_id];
 	}
 
 	if (!regd) {
@@ -279,6 +278,7 @@ ath11k_map_fw_dfs_region(enum ath11k_dfs_region dfs_region)
 	case ATH11K_DFS_REG_KR:
 		return NL80211_DFS_ETSI;
 	case ATH11K_DFS_REG_MKK:
+	case ATH11K_DFS_REG_MKK_N:
 		return NL80211_DFS_JP;
 	default:
 		return NL80211_DFS_UNSET;
@@ -456,9 +456,6 @@ ath11k_reg_adjust_bw(u16 start_freq, u16 end_freq, u16 max_bw)
 {
 	u16 bw;
 
-	if (end_freq <= start_freq)
-		return 0;
-
 	bw = end_freq - start_freq;
 	bw = min_t(u16, bw, max_bw);
 
@@ -466,10 +463,8 @@ ath11k_reg_adjust_bw(u16 start_freq, u16 end_freq, u16 max_bw)
 		bw = 80;
 	else if (bw >= 40 && bw < 80)
 		bw = 40;
-	else if (bw >= 20 && bw < 40)
+	else if (bw < 40)
 		bw = 20;
-	else
-		bw = 0;
 
 	return bw;
 }
@@ -493,77 +488,73 @@ ath11k_reg_update_weather_radar_band(struct ath11k_base *ab,
 				     struct cur_reg_rule *reg_rule,
 				     u8 *rule_idx, u32 flags, u16 max_bw)
 {
-	u32 start_freq;
 	u32 end_freq;
 	u16 bw;
 	u8 i;
 
 	i = *rule_idx;
 
-	/* there might be situations when even the input rule must be dropped */
-	i--;
-
-	/* frequencies below weather radar */
 	bw = ath11k_reg_adjust_bw(reg_rule->start_freq,
 				  ETSI_WEATHER_RADAR_BAND_LOW, max_bw);
-	if (bw > 0) {
-		i++;
 
-		ath11k_reg_update_rule(regd->reg_rules + i,
-				       reg_rule->start_freq,
-				       ETSI_WEATHER_RADAR_BAND_LOW, bw,
-				       reg_rule->ant_gain, reg_rule->reg_power,
-				       flags);
+	ath11k_reg_update_rule(regd->reg_rules + i, reg_rule->start_freq,
+			       ETSI_WEATHER_RADAR_BAND_LOW, bw,
+			       reg_rule->ant_gain, reg_rule->reg_power,
+			       flags);
 
-		ath11k_dbg(ab, ATH11K_DBG_REG,
-			   "\t%d. (%d - %d @ %d) (%d, %d) (%d ms) (FLAGS %d)\n",
-			   i + 1, reg_rule->start_freq,
-			   ETSI_WEATHER_RADAR_BAND_LOW, bw, reg_rule->ant_gain,
-			   reg_rule->reg_power, regd->reg_rules[i].dfs_cac_ms,
-			   flags);
+	ath11k_dbg(ab, ATH11K_DBG_REG,
+		   "\t%d. (%d - %d @ %d) (%d, %d) (%d ms) (FLAGS %d)\n",
+		   i + 1, reg_rule->start_freq, ETSI_WEATHER_RADAR_BAND_LOW,
+		   bw, reg_rule->ant_gain, reg_rule->reg_power,
+		   regd->reg_rules[i].dfs_cac_ms,
+		   flags);
+
+	if (reg_rule->end_freq > ETSI_WEATHER_RADAR_BAND_HIGH)
+		end_freq = ETSI_WEATHER_RADAR_BAND_HIGH;
+	else
+		end_freq = reg_rule->end_freq;
+
+	bw = ath11k_reg_adjust_bw(ETSI_WEATHER_RADAR_BAND_LOW, end_freq,
+				  max_bw);
+
+	i++;
+
+	ath11k_reg_update_rule(regd->reg_rules + i,
+			       ETSI_WEATHER_RADAR_BAND_LOW, end_freq, bw,
+			       reg_rule->ant_gain, reg_rule->reg_power,
+			       flags);
+
+	regd->reg_rules[i].dfs_cac_ms = ETSI_WEATHER_RADAR_BAND_CAC_TIMEOUT;
+
+	ath11k_dbg(ab, ATH11K_DBG_REG,
+		   "\t%d. (%d - %d @ %d) (%d, %d) (%d ms) (FLAGS %d)\n",
+		   i + 1, ETSI_WEATHER_RADAR_BAND_LOW, end_freq,
+		   bw, reg_rule->ant_gain, reg_rule->reg_power,
+		   regd->reg_rules[i].dfs_cac_ms,
+		   flags);
+
+	if (end_freq == reg_rule->end_freq) {
+		regd->n_reg_rules--;
+		*rule_idx = i;
+		return;
 	}
 
-	/* weather radar frequencies */
-	start_freq = max_t(u32, reg_rule->start_freq,
-			   ETSI_WEATHER_RADAR_BAND_LOW);
-	end_freq = min_t(u32, reg_rule->end_freq, ETSI_WEATHER_RADAR_BAND_HIGH);
-
-	bw = ath11k_reg_adjust_bw(start_freq, end_freq, max_bw);
-	if (bw > 0) {
-		i++;
-
-		ath11k_reg_update_rule(regd->reg_rules + i, start_freq,
-				       end_freq, bw, reg_rule->ant_gain,
-				       reg_rule->reg_power, flags);
-
-		regd->reg_rules[i].dfs_cac_ms = ETSI_WEATHER_RADAR_BAND_CAC_TIMEOUT;
-
-		ath11k_dbg(ab, ATH11K_DBG_REG,
-			   "\t%d. (%d - %d @ %d) (%d, %d) (%d ms) (FLAGS %d)\n",
-			   i + 1, start_freq, end_freq, bw,
-			   reg_rule->ant_gain, reg_rule->reg_power,
-			   regd->reg_rules[i].dfs_cac_ms, flags);
-	}
-
-	/* frequencies above weather radar */
 	bw = ath11k_reg_adjust_bw(ETSI_WEATHER_RADAR_BAND_HIGH,
 				  reg_rule->end_freq, max_bw);
-	if (bw > 0) {
-		i++;
 
-		ath11k_reg_update_rule(regd->reg_rules + i,
-				       ETSI_WEATHER_RADAR_BAND_HIGH,
-				       reg_rule->end_freq, bw,
-				       reg_rule->ant_gain, reg_rule->reg_power,
-				       flags);
+	i++;
 
-		ath11k_dbg(ab, ATH11K_DBG_REG,
-			   "\t%d. (%d - %d @ %d) (%d, %d) (%d ms) (FLAGS %d)\n",
-			   i + 1, ETSI_WEATHER_RADAR_BAND_HIGH,
-			   reg_rule->end_freq, bw, reg_rule->ant_gain,
-			   reg_rule->reg_power, regd->reg_rules[i].dfs_cac_ms,
-			   flags);
-	}
+	ath11k_reg_update_rule(regd->reg_rules + i, ETSI_WEATHER_RADAR_BAND_HIGH,
+			       reg_rule->end_freq, bw,
+			       reg_rule->ant_gain, reg_rule->reg_power,
+			       flags);
+
+	ath11k_dbg(ab, ATH11K_DBG_REG,
+		   "\t%d. (%d - %d @ %d) (%d, %d) (%d ms) (FLAGS %d)\n",
+		   i + 1, ETSI_WEATHER_RADAR_BAND_HIGH, reg_rule->end_freq,
+		   bw, reg_rule->ant_gain, reg_rule->reg_power,
+		   regd->reg_rules[i].dfs_cac_ms,
+		   flags);
 
 	*rule_idx = i;
 }
@@ -690,7 +681,7 @@ void ath11k_regd_update_work(struct work_struct *work)
 					 regd_update_work);
 	int ret;
 
-	ret = ath11k_regd_update(ar);
+	ret = ath11k_regd_update(ar, false);
 	if (ret) {
 		/* Firmware has already moved to the new regd. We need
 		 * to maintain channel consistency across FW, Host driver
